@@ -18,37 +18,27 @@
 
 import sys
 
-from oslo_concurrency import processutils
-from oslo_log import log as logging
-from oslo_reports import guru_meditation_report as gmr
-from oslo_reports import opts as gmr_opts
+from oslo.config import cfg
 
-import nova.conf
-from nova.conf import remote_debug
 from nova import config
-from nova import objects
-from nova.scheduler import rpcapi
+from nova.openstack.common import log as logging
+from nova.openstack.common.report import guru_meditation_report as gmr
 from nova import service
+from nova import utils
 from nova import version
 
-CONF = nova.conf.CONF
-remote_debug.register_cli_opts(CONF)
+CONF = cfg.CONF
+CONF.import_opt('scheduler_topic', 'nova.scheduler.rpcapi')
 
 
 def main():
     config.parse_args(sys.argv)
-    logging.setup(CONF, "nova")
-    objects.register_all()
-    gmr_opts.set_defaults(CONF)
-    objects.Service.enable_min_version_cache()
+    logging.setup("nova")
+    utils.monkey_patch()
 
-    gmr.TextGuruMeditation.setup_autorun(version, conf=CONF)
+    gmr.TextGuruMeditation.setup_autorun(version)
 
-    server = service.Service.create(
-        binary='nova-scheduler', topic=rpcapi.RPC_TOPIC)
-
-    # Determine the number of workers; if not specified in config, default
-    # to number of CPUs
-    workers = CONF.scheduler.workers or processutils.get_worker_count()
-    service.serve(server, workers=workers)
+    server = service.Service.create(binary='nova-scheduler',
+                                    topic=CONF.scheduler_topic)
+    service.serve(server)
     service.wait()
