@@ -15,21 +15,25 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-from oslo_concurrency import processutils
-from oslo_log import log as logging
-import oslo_messaging as messaging
-from oslo_utils import excutils
-from oslo_utils import importutils
-from oslo_utils import uuidutils
-import six
+from oslo.config import cfg
+from oslo import messaging
 
-import nova.conf
 from nova import context
 from nova.db import base
 from nova import exception
-from nova.i18n import _LE, _LI, _LW
 from nova.network import rpcapi as network_rpcapi
-from nova import objects
+from nova.objects import dns_domain as dns_domain_obj
+from nova.objects import fixed_ip as fixed_ip_obj
+from nova.objects import floating_ip as floating_ip_obj
+from nova.objects import instance as instance_obj
+from nova.objects import network as network_obj
+from nova.objects import service as service_obj
+from nova.openstack.common import excutils
+from nova.openstack.common.gettextutils import _
+from nova.openstack.common import importutils
+from nova.openstack.common import log as logging
+from nova.openstack.common import processutils
+from nova.openstack.common import uuidutils
 from nova import quota
 from nova import rpc
 from nova import servicegroup
@@ -39,7 +43,28 @@ LOG = logging.getLogger(__name__)
 
 QUOTAS = quota.QUOTAS
 
-CONF = nova.conf.CONF
+floating_opts = [
+    cfg.StrOpt('default_floating_pool',
+               default='nova',
+               help='Default pool for floating IPs'),
+    cfg.BoolOpt('auto_assign_floating_ip',
+                default=False,
+                help='Autoassigning floating IP to VM'),
+    cfg.StrOpt('floating_ip_dns_manager',
+               default='nova.network.noop_dns_driver.NoopDNSDriver',
+               help='Full class name for the DNS Manager for floating IPs'),
+    cfg.StrOpt('instance_dns_manager',
+               default='nova.network.noop_dns_driver.NoopDNSDriver',
+               help='Full class name for the DNS Manager for instance IPs'),
+    cfg.StrOpt('instance_dns_domain',
+               default='',
+               help='Full class name for the DNS Zone for instance IPs'),
+]
+
+CONF = cfg.CONF
+CONF.register_opts(floating_opts)
+CONF.import_opt('public_interface', 'nova.network.linux_net')
+CONF.import_opt('network_topic', 'nova.network.rpcapi')
 
 
 class FloatingIP(object):
@@ -48,12 +73,12 @@ class FloatingIP(object):
     servicegroup_api = None
 
     def init_host_floating_ips(self):
-        """Configures floating IPs owned by host."""
+        """Configures floating ips owned by host."""
 
         admin_context = context.get_admin_context()
         try:
-            floating_ips = objects.FloatingIPList.get_by_host(admin_context,
-                                                              self.host)
+            floating_ips = floating_ip_obj.FloatingIPList.get_by_host(
+                admin_context, self.host)
         except exception.NotFound:
             return
 
@@ -62,7 +87,8 @@ class FloatingIP(object):
                 try:
                     fixed_ip = floating_ip.fixed_ip
                 except exception.FixedIpNotFound:
-                    LOG.debug('Fixed IP %s not found', floating_ip.fixed_ip_id)
+                    msg = _('Fixed ip %s not found') % floating_ip.fixed_ip_id
+                    LOG.debug(msg)
                     continue
                 interface = CONF.public_interface or floating_ip.interface
                 try:
@@ -71,7 +97,7 @@ class FloatingIP(object):
                                                   interface,
                                                   fixed_ip.network)
                 except processutils.ProcessExecutionError:
-                    LOG.debug('Interface %s not found', interface)
+                    LOG.debug(_('Interface %s not found'), interface)
                     raise exception.NoFloatingIpInterface(interface=interface)
 
     def allocate_for_instance(self, context, **kwargs):
@@ -85,19 +111,19 @@ class FloatingIP(object):
         if not uuidutils.is_uuid_like(instance_uuid):
             instance_uuid = kwargs.get('instance_uuid')
         project_id = kwargs.get('project_id')
+        requested_networks = kwargs.get('requested_networks')
         # call the next inherited class's allocate_for_instance()
         # which is currently the NetworkManager version
         # do this first so fixed ip is already allocated
         nw_info = super(FloatingIP, self).allocate_for_instance(context,
                                                                 **kwargs)
         if CONF.auto_assign_floating_ip:
-            context = context.elevated()
             # allocate a floating ip
             floating_address = self.allocate_floating_ip(context, project_id,
                 True)
-            LOG.debug("floating IP allocation for instance "
-                      "|%s|", floating_address,
-                      instance_uuid=instance_uuid, context=context)
+            LOG.debug(_("floating IP allocation for instance "
+                        "|%s|"), floating_address,
+                        instance_uuid=instance_uuid, context=context)
 
             # get the first fixed address belonging to the instance
             fixed_ips = nw_info.fixed_ips()
@@ -129,12 +155,12 @@ class FloatingIP(object):
                 # NOTE(francois.charlier): in some cases the instance might be
                 # deleted before the IPs are released, so we need to get
                 # deleted instances too
-                instance = objects.Instance.get_by_id(
+                instance = instance_obj.Instance.get_by_id(
                     context.elevated(read_deleted='yes'), instance_uuid)
                 instance_uuid = instance.uuid
 
         try:
-            fixed_ips = objects.FixedIPList.get_by_instance_uuid(
+            fixed_ips = fixed_ip_obj.FixedIPList.get_by_instance_uuid(
                 context, instance_uuid)
         except exception.FixedIpNotFoundForInstance:
             fixed_ips = []
@@ -142,8 +168,8 @@ class FloatingIP(object):
         kwargs['fixed_ips'] = fixed_ips
         for fixed_ip in fixed_ips:
             fixed_id = fixed_ip.id
-            floating_ips = objects.FloatingIPList.get_by_fixed_ip_id(context,
-                                                                     fixed_id)
+            floating_ips = floating_ip_obj.FloatingIPList.get_by_fixed_ip_id(
+                context, fixed_id)
             # disassociate floating ips related to fixed_ip
             for floating_ip in floating_ips:
                 address = str(floating_ip.address)
@@ -152,7 +178,7 @@ class FloatingIP(object):
                                                   address,
                                                   affect_auto_assigned=True)
                 except exception.FloatingIpNotAssociated:
-                    LOG.info(_LI("Floating IP %s is not associated. Ignore."),
+                    LOG.info(_("Floating IP %s is not associated. Ignore."),
                              address)
                 # deallocate if auto_assigned
                 if floating_ip.auto_assigned:
@@ -165,43 +191,29 @@ class FloatingIP(object):
         super(FloatingIP, self).deallocate_for_instance(context, **kwargs)
 
     def _floating_ip_owned_by_project(self, context, floating_ip):
-        """Raises if floating IP does not belong to project."""
+        """Raises if floating ip does not belong to project."""
         if context.is_admin:
             return
 
         if floating_ip.project_id != context.project_id:
             if floating_ip.project_id is None:
-                LOG.warning(_LW('Address |%(address)s| is not allocated'),
-                            {'address': floating_ip.address})
-                raise exception.Forbidden()
+                LOG.warn(_('Address |%(address)s| is not allocated'),
+                           {'address': floating_ip.address})
+                raise exception.NotAuthorized()
             else:
-                LOG.warning(_LW('Address |%(address)s| is not allocated '
-                                'to your project |%(project)s|'),
-                            {'address': floating_ip.address,
-                             'project': context.project_id})
-                raise exception.Forbidden()
-
-    def _floating_ip_pool_exists(self, context, name):
-        """Returns true if the specified floating IP pool exists. Otherwise,
-        returns false.
-        """
-        pools = [pool.get('name') for pool in
-                 self.get_floating_ip_pools(context)]
-        if name in pools:
-            return True
-
-        return False
+                LOG.warn(_('Address |%(address)s| is not allocated to your '
+                           'project |%(project)s|'),
+                           {'address': floating_ip.address,
+                           'project': context.project_id})
+                raise exception.NotAuthorized()
 
     def allocate_floating_ip(self, context, project_id, auto_assigned=False,
                              pool=None):
-        """Gets a floating IP from the pool."""
+        """Gets a floating ip from the pool."""
         # NOTE(tr3buchet): all network hosts in zone now use the same pool
         pool = pool or CONF.default_floating_pool
         use_quota = not auto_assigned
 
-        if not self._floating_ip_pool_exists(context, pool):
-            raise exception.FloatingIpPoolNotFound()
-
         # Check the quota; can't put this in the API because we get
         # called into from other places
         try:
@@ -209,12 +221,12 @@ class FloatingIP(object):
                 reservations = QUOTAS.reserve(context, floating_ips=1,
                                               project_id=project_id)
         except exception.OverQuota:
-            LOG.warning(_LW("Quota exceeded for %s, tried to allocate "
-                            "floating IP"), context.project_id)
+            LOG.warn(_("Quota exceeded for %s, tried to allocate "
+                       "floating IP"), context.project_id)
             raise exception.FloatingIpLimitExceeded()
 
         try:
-            floating_ip = objects.FloatingIP.allocate_address(
+            floating_ip = floating_ip_obj.FloatingIP.allocate_address(
                 context, project_id, pool, auto_assigned=auto_assigned)
             payload = dict(project_id=project_id, floating_ip=floating_ip)
             self.notifier.info(context,
@@ -234,8 +246,9 @@ class FloatingIP(object):
     @messaging.expected_exceptions(exception.FloatingIpNotFoundForAddress)
     def deallocate_floating_ip(self, context, address,
                                affect_auto_assigned=False):
-        """Returns a floating IP to the pool."""
-        floating_ip = objects.FloatingIP.get_by_address(context, address)
+        """Returns a floating ip to the pool."""
+        floating_ip = floating_ip_obj.FloatingIP.get_by_address(context,
+                                                                address)
 
         # handle auto_assigned
         if not affect_auto_assigned and floating_ip.auto_assigned:
@@ -268,35 +281,30 @@ class FloatingIP(object):
                 reservations = None
         except Exception:
             reservations = None
-            LOG.exception(_LE("Failed to update usages deallocating "
-                              "floating IP"))
+            LOG.exception(_("Failed to update usages deallocating "
+                            "floating IP"))
 
-        rows_updated = objects.FloatingIP.deallocate(context, address)
-        # number of updated rows will be 0 if concurrently another
-        # API call has also deallocated the same floating ip
-        if not rows_updated:
-            if reservations:
-                QUOTAS.rollback(context, reservations, project_id=project_id)
-        else:
-            # Commit the reservations
-            if reservations:
-                QUOTAS.commit(context, reservations, project_id=project_id)
+        floating_ip_obj.FloatingIP.deallocate(context, address)
+
+        # Commit the reservations
+        if reservations:
+            QUOTAS.commit(context, reservations, project_id=project_id)
 
     @messaging.expected_exceptions(exception.FloatingIpNotFoundForAddress)
     def associate_floating_ip(self, context, floating_address, fixed_address,
                               affect_auto_assigned=False):
-        """Associates a floating IP with a fixed IP.
+        """Associates a floating ip with a fixed ip.
 
         Makes sure everything makes sense then calls _associate_floating_ip,
         rpc'ing to correct host if i'm not it.
 
         Access to the floating_address is verified but access to the
-        fixed_address is not verified. This assumes that the calling
+        fixed_address is not verified. This assumes that that the calling
         side has already verified that the fixed_address is legal by
         checking access to the instance.
         """
-        floating_ip = objects.FloatingIP.get_by_address(context,
-                                                        floating_address)
+        floating_ip = floating_ip_obj.FloatingIP.get_by_address(
+            context, floating_address)
         # handle auto_assigned
         if not affect_auto_assigned and floating_ip.auto_assigned:
             return
@@ -316,13 +324,14 @@ class FloatingIP(object):
 
             self.disassociate_floating_ip(context, floating_address)
 
-        fixed_ip = objects.FixedIP.get_by_address(context, fixed_address)
+        fixed_ip = fixed_ip_obj.FixedIP.get_by_address(context,
+                                                       fixed_address)
 
         # send to correct host, unless i'm the correct host
-        network = objects.Network.get_by_id(context.elevated(),
-                                            fixed_ip.network_id)
+        network = network_obj.Network.get_by_id(context.elevated(),
+                                                fixed_ip.network_id)
         if network.multi_host:
-            instance = objects.Instance.get_by_uuid(
+            instance = instance_obj.Instance.get_by_uuid(
                 context, fixed_ip.instance_uuid)
             host = instance.host
         else:
@@ -344,14 +353,16 @@ class FloatingIP(object):
 
     def _associate_floating_ip(self, context, floating_address, fixed_address,
                                interface, instance_uuid):
-        """Performs db and driver calls to associate floating IP & fixed IP."""
+        """Performs db and driver calls to associate floating ip & fixed ip."""
         interface = CONF.public_interface or interface
 
-        @utils.synchronized(six.text_type(floating_address))
+        @utils.synchronized(unicode(floating_address))
         def do_associate():
             # associate floating ip
-            floating = objects.FloatingIP.associate(context, floating_address,
-                                                    fixed_address, self.host)
+            floating = floating_ip_obj.FloatingIP.associate(context,
+                                                            floating_address,
+                                                            fixed_address,
+                                                            self.host)
             fixed = floating.fixed_ip
             if not fixed:
                 # NOTE(vish): ip was already associated
@@ -361,17 +372,17 @@ class FloatingIP(object):
                 self.l3driver.add_floating_ip(floating_address, fixed_address,
                         interface, fixed['network'])
             except processutils.ProcessExecutionError as e:
-                with excutils.save_and_reraise_exception():
+                with excutils.save_and_reraise_exception() as exc_ctxt:
                     try:
-                        objects.FloatingIP.disassociate(context,
-                                                        floating_address)
+                        floating_ip_obj.FloatingIP.disassociate(
+                            context, floating_address)
                     except Exception:
-                        LOG.warning(_LW('Failed to disassociated floating '
-                                        'address: %s'), floating_address)
+                        LOG.warn(_('Failed to disassociated floating '
+                                   'address: %s'), floating_address)
                         pass
-                    if "Cannot find device" in six.text_type(e):
+                    if "Cannot find device" in str(e):
                         try:
-                            LOG.error(_LE('Interface %s not found'), interface)
+                            LOG.error(_('Interface %s not found'), interface)
                         except Exception:
                             pass
                         raise exception.NoFloatingIpInterface(
@@ -387,12 +398,13 @@ class FloatingIP(object):
     @messaging.expected_exceptions(exception.FloatingIpNotFoundForAddress)
     def disassociate_floating_ip(self, context, address,
                                  affect_auto_assigned=False):
-        """Disassociates a floating IP from its fixed IP.
+        """Disassociates a floating ip from its fixed ip.
 
         Makes sure everything makes sense then calls _disassociate_floating_ip,
         rpc'ing to correct host if i'm not it.
         """
-        floating_ip = objects.FloatingIP.get_by_address(context, address)
+        floating_ip = floating_ip_obj.FloatingIP.get_by_address(context,
+                                                                address)
 
         # handle auto assigned
         if not affect_auto_assigned and floating_ip.auto_assigned:
@@ -406,17 +418,18 @@ class FloatingIP(object):
             floating_address = floating_ip.address
             raise exception.FloatingIpNotAssociated(address=floating_address)
 
-        fixed_ip = objects.FixedIP.get_by_id(context, floating_ip.fixed_ip_id)
+        fixed_ip = fixed_ip_obj.FixedIP.get_by_id(context,
+                                                  floating_ip.fixed_ip_id)
 
         # send to correct host, unless i'm the correct host
-        network = objects.Network.get_by_id(context.elevated(),
-                                            fixed_ip.network_id)
+        network = network_obj.Network.get_by_id(context.elevated(),
+                                                fixed_ip.network_id)
         interface = floating_ip.interface
         if network.multi_host:
-            instance = objects.Instance.get_by_uuid(
+            instance = instance_obj.Instance.get_by_uuid(
                 context, fixed_ip.instance_uuid)
-            service = objects.Service.get_by_host_and_binary(
-                context.elevated(), instance.host, 'nova-network')
+            service = service_obj.Service.get_by_host_and_topic(
+                context.elevated(), instance.host, CONF.network_topic)
             if service and self.servicegroup_api.service_is_up(service):
                 host = instance.host
             else:
@@ -440,10 +453,10 @@ class FloatingIP(object):
 
     def _disassociate_floating_ip(self, context, address, interface,
                                   instance_uuid):
-        """Performs db and driver calls to disassociate floating IP."""
+        """Performs db and driver calls to disassociate floating ip."""
         interface = CONF.public_interface or interface
 
-        @utils.synchronized(six.text_type(address))
+        @utils.synchronized(unicode(address))
         def do_disassociate():
             # NOTE(vish): Note that we are disassociating in the db before we
             #             actually remove the ip address on the host. We are
@@ -452,7 +465,8 @@ class FloatingIP(object):
             #             don't worry about this case because the minuscule
             #             window where the ip is on both hosts shouldn't cause
             #             any problems.
-            floating = objects.FloatingIP.disassociate(context, address)
+            floating = floating_ip_obj.FloatingIP.disassociate(context,
+                                                               address)
             fixed = floating.fixed_ip
             if not fixed:
                 # NOTE(vish): ip was already disassociated
@@ -473,7 +487,8 @@ class FloatingIP(object):
         """Returns a floating IP as a dict."""
         # NOTE(vish): This is no longer used but can't be removed until
         #             we major version the network_rpcapi.
-        return dict(objects.FloatingIP.get_by_id(context, id))
+        return dict(floating_ip_obj.FloatingIP.get_by_id(
+                context, id).iteritems())
 
     def get_floating_pools(self, context):
         """Returns list of floating pools."""
@@ -485,34 +500,39 @@ class FloatingIP(object):
         """Returns list of floating ip pools."""
         # NOTE(vish): This is no longer used but can't be removed until
         #             we major version the network_rpcapi.
-        pools = objects.FloatingIP.get_pool_names(context)
+        pools = floating_ip_obj.FloatingIP.get_pool_names(context)
         return [dict(name=name) for name in pools]
 
     def get_floating_ip_by_address(self, context, address):
         """Returns a floating IP as a dict."""
         # NOTE(vish): This is no longer used but can't be removed until
         #             we major version the network_rpcapi.
-        return objects.FloatingIP.get_by_address(context, address)
+        # NOTE(danms): Not converting to objects since it's not used
+        return dict(self.db.floating_ip_get_by_address(context,
+                                                       address).iteritems())
 
     def get_floating_ips_by_project(self, context):
         """Returns the floating IPs allocated to a project."""
         # NOTE(vish): This is no longer used but can't be removed until
         #             we major version the network_rpcapi.
-        return objects.FloatingIPList.get_by_project(context,
+        # NOTE(danms): Not converting to objects since it's not used
+        ips = self.db.floating_ip_get_all_by_project(context,
                                                      context.project_id)
+        return [dict(ip.iteritems()) for ip in ips]
 
     def get_floating_ips_by_fixed_address(self, context, fixed_address):
         """Returns the floating IPs associated with a fixed_address."""
         # NOTE(vish): This is no longer used but can't be removed until
         #             we major version the network_rpcapi.
-        floating_ips = objects.FloatingIPList.get_by_fixed_address(
-            context, fixed_address)
-        return [str(floating_ip.address) for floating_ip in floating_ips]
+        # NOTE(danms): Not converting to objects since it's not used
+        floating_ips = self.db.floating_ip_get_by_fixed_address(context,
+                                                                fixed_address)
+        return [floating_ip['address'] for floating_ip in floating_ips]
 
     def _is_stale_floating_ip_address(self, context, floating_ip):
         try:
             self._floating_ip_owned_by_project(context, floating_ip)
-        except exception.Forbidden:
+        except exception.NotAuthorized:
             return True
         return False if floating_ip.get('fixed_ip_id') else True
 
@@ -525,17 +545,17 @@ class FloatingIP(object):
         if not floating_addresses or (source and source == dest):
             return
 
-        LOG.info(_LI("Starting migration network for instance %s"),
+        LOG.info(_("Starting migration network for instance %s"),
                  instance_uuid)
         for address in floating_addresses:
-            floating_ip = objects.FloatingIP.get_by_address(context, address)
+            floating_ip = floating_ip_obj.FloatingIP.get_by_address(context,
+                                                                    address)
 
             if self._is_stale_floating_ip_address(context, floating_ip):
-                LOG.warning(_LW("Floating IP address |%(address)s| no longer "
-                                "belongs to instance %(instance_uuid)s. "
-                                "Will not migrate it "),
-                            {'address': address,
-                             'instance_uuid': instance_uuid})
+                LOG.warn(_("Floating ip address |%(address)s| no longer "
+                           "belongs to instance %(instance_uuid)s. Will not "
+                           "migrate it "),
+                         {'address': address, 'instance_uuid': instance_uuid})
                 continue
 
             interface = CONF.public_interface or floating_ip.interface
@@ -545,6 +565,10 @@ class FloatingIP(object):
                                              interface,
                                              fixed_ip.network)
 
+            # NOTE(ivoks): Destroy conntrack entries on source compute
+            # host.
+            self.l3driver.clean_conntrack(fixed_ip.address)
+
             # NOTE(wenjianhn): Make this address will not be bound to public
             # interface when restarts nova-network on dest compute node
             floating_ip.host = None
@@ -561,18 +585,18 @@ class FloatingIP(object):
         if not floating_addresses or (source and source == dest):
             return
 
-        LOG.info(_LI("Finishing migration network for instance %s"),
+        LOG.info(_("Finishing migration network for instance %s"),
                  instance_uuid)
 
         for address in floating_addresses:
-            floating_ip = objects.FloatingIP.get_by_address(context, address)
+            floating_ip = floating_ip_obj.FloatingIP.get_by_address(context,
+                                                                    address)
 
             if self._is_stale_floating_ip_address(context, floating_ip):
-                LOG.warning(_LW("Floating IP address |%(address)s| no longer "
-                                "belongs to instance %(instance_uuid)s. "
-                                "Will not setup it."),
-                            {'address': address,
-                             'instance_uuid': instance_uuid})
+                LOG.warn(_("Floating ip address |%(address)s| no longer "
+                           "belongs to instance %(instance_uuid)s. Will not"
+                           "setup it."),
+                         {'address': address, 'instance_uuid': instance_uuid})
                 continue
 
             floating_ip.host = dest
@@ -600,21 +624,22 @@ class FloatingIP(object):
     def get_dns_domains(self, context):
         domains = []
 
-        domain_list = objects.DNSDomainList.get_all(context)
+        domain_list = dns_domain_obj.DNSDomainList.get_all(context)
         floating_driver_domain_list = self.floating_dns_manager.get_domains()
         instance_driver_domain_list = self.instance_dns_manager.get_domains()
 
         for dns_domain in domain_list:
             if (dns_domain.domain in floating_driver_domain_list or
                     dns_domain.domain in instance_driver_domain_list):
-                domain_entry = self._prepare_domain_entry(context, dns_domain)
-                if domain_entry:
-                    domains.append(domain_entry)
+                    domain_entry = self._prepare_domain_entry(context,
+                                                              dns_domain)
+                    if domain_entry:
+                        domains.append(domain_entry)
             else:
-                LOG.warning(_LW('Database inconsistency: DNS domain |%s| is '
-                                'registered in the Nova db but not visible to '
-                                'either the floating or instance DNS driver. '
-                                'It will be ignored.'), dns_domain.domain)
+                LOG.warn(_('Database inconsistency: DNS domain |%s| is '
+                         'registered in the Nova db but not visible to '
+                         'either the floating or instance DNS driver. It '
+                         'will be ignored.'), dns_domain.domain)
 
         return domains
 
@@ -647,25 +672,25 @@ class FloatingIP(object):
                                                              domain)
 
     def create_private_dns_domain(self, context, domain, av_zone):
-        objects.DNSDomain.register_for_zone(context, domain, av_zone)
+        dns_domain_obj.DNSDomain.register_for_zone(context, domain, av_zone)
         try:
             self.instance_dns_manager.create_domain(domain)
         except exception.FloatingIpDNSExists:
-            LOG.warning(_LW('Domain |%(domain)s| already exists, '
-                            'changing zone to |%(av_zone)s|.'),
-                        {'domain': domain, 'av_zone': av_zone})
+            LOG.warn(_('Domain |%(domain)s| already exists, '
+                       'changing zone to |%(av_zone)s|.'),
+                     {'domain': domain, 'av_zone': av_zone})
 
     def create_public_dns_domain(self, context, domain, project):
-        objects.DNSDomain.register_for_project(context, domain, project)
+        dns_domain_obj.DNSDomain.register_for_project(context, domain, project)
         try:
             self.floating_dns_manager.create_domain(domain)
         except exception.FloatingIpDNSExists:
-            LOG.warning(_LW('Domain |%(domain)s| already exists, '
-                            'changing project to |%(project)s|.'),
-                        {'domain': domain, 'project': project})
+            LOG.warn(_('Domain |%(domain)s| already exists, '
+                       'changing project to |%(project)s|.'),
+                     {'domain': domain, 'project': project})
 
     def delete_dns_domain(self, context, domain):
-        objects.DNSDomain.delete_by_domain(context, domain)
+        dns_domain_obj.DNSDomain.delete_by_domain(context, domain)
         self.floating_dns_manager.delete_domain(domain)
 
 
