@@ -17,35 +17,41 @@ Image caching and management.
 """
 import os
 
-from os_win import utilsfactory
-from oslo_log import log as logging
-from oslo_utils import excutils
-from oslo_utils import units
+from oslo.config import cfg
 
-import nova.conf
-from nova import exception
+from nova.compute import flavors
+from nova.openstack.common import excutils
+from nova.openstack.common.gettextutils import _
+from nova.openstack.common import log as logging
+from nova.openstack.common import units
 from nova import utils
-from nova.virt.hyperv import pathutils
+from nova.virt.hyperv import utilsfactory
+from nova.virt.hyperv import vmutils
 from nova.virt import images
 
 LOG = logging.getLogger(__name__)
 
-CONF = nova.conf.CONF
+CONF = cfg.CONF
+CONF.import_opt('use_cow_images', 'nova.virt.driver')
 
 
 class ImageCache(object):
     def __init__(self):
-        self._pathutils = pathutils.PathUtils()
+        self._pathutils = utilsfactory.get_pathutils()
         self._vhdutils = utilsfactory.get_vhdutils()
 
     def _get_root_vhd_size_gb(self, instance):
-        if instance.old_flavor:
-            return instance.old_flavor.root_gb
-        else:
-            return instance.root_gb
+        try:
+            # In case of resizes we need the old root disk size
+            old_flavor = flavors.extract_flavor(
+                instance, prefix='old_')
+            return old_flavor['root_gb']
+        except KeyError:
+            return instance['root_gb']
 
     def _resize_and_cache_vhd(self, instance, vhd_path):
-        vhd_size = self._vhdutils.get_vhd_size(vhd_path)['VirtualSize']
+        vhd_info = self._vhdutils.get_vhd_info(vhd_path)
+        vhd_size = vhd_info['MaxInternalSize']
 
         root_vhd_size_gb = self._get_root_vhd_size_gb(instance)
         root_vhd_size = root_vhd_size_gb * units.Gi
@@ -55,8 +61,12 @@ class ImageCache(object):
                     vhd_path, root_vhd_size))
 
         if root_vhd_internal_size < vhd_size:
-            raise exception.FlavorDiskSmallerThanImage(
-                flavor_size=root_vhd_size, image_size=vhd_size)
+            raise vmutils.HyperVException(
+                _("Cannot resize the image to a size smaller than the VHD "
+                  "max. internal size: %(vhd_size)s. Requested disk size: "
+                  "%(root_vhd_size)s") %
+                {'vhd_size': vhd_size, 'root_vhd_size': root_vhd_size}
+            )
         if root_vhd_internal_size > vhd_size:
             path_parts = os.path.splitext(vhd_path)
             resized_vhd_path = '%s_%s%s' % (path_parts[0],
@@ -67,13 +77,13 @@ class ImageCache(object):
             def copy_and_resize_vhd():
                 if not self._pathutils.exists(resized_vhd_path):
                     try:
-                        LOG.debug("Copying VHD %(vhd_path)s to "
-                                  "%(resized_vhd_path)s",
+                        LOG.debug(_("Copying VHD %(vhd_path)s to "
+                                    "%(resized_vhd_path)s"),
                                   {'vhd_path': vhd_path,
                                    'resized_vhd_path': resized_vhd_path})
                         self._pathutils.copyfile(vhd_path, resized_vhd_path)
-                        LOG.debug("Resizing VHD %(resized_vhd_path)s to new "
-                                  "size %(root_vhd_size)s",
+                        LOG.debug(_("Resizing VHD %(resized_vhd_path)s to new "
+                                    "size %(root_vhd_size)s"),
                                   {'resized_vhd_path': resized_vhd_path,
                                    'root_vhd_size': root_vhd_size})
                         self._vhdutils.resize_vhd(resized_vhd_path,
@@ -88,7 +98,7 @@ class ImageCache(object):
             return resized_vhd_path
 
     def get_cached_image(self, context, instance):
-        image_id = instance.image_ref
+        image_id = instance['image_ref']
 
         base_vhd_dir = self._pathutils.get_base_vhd_dir()
         base_vhd_path = os.path.join(base_vhd_dir, image_id)
@@ -105,8 +115,8 @@ class ImageCache(object):
             if not vhd_path:
                 try:
                     images.fetch(context, image_id, base_vhd_path,
-                                 instance.user_id,
-                                 instance.project_id)
+                                 instance['user_id'],
+                                 instance['project_id'])
 
                     format_ext = self._vhdutils.get_vhd_format(base_vhd_path)
                     vhd_path = base_vhd_path + '.' + format_ext.lower()
