@@ -14,21 +14,24 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-import oslo_config.cfg
-from oslo_utils import importutils
+import oslo.config.cfg
+
+# Importing full names to not pollute the namespace and cause possible
+# collisions with use of 'from nova.volume import <foo>' elsewhere.
+import nova.openstack.common.importutils
 
 _volume_opts = [
-    oslo_config.cfg.StrOpt(
-        'volume_api_class',
-        default='nova.volume.cinder.API',
-        help='DEPRECATED: The full class name of the volume API class to use',
-        deprecated_for_removal=True)
+    oslo.config.cfg.StrOpt('volume_api_class',
+                                     default='nova.volume.cinder.API',
+                                     help='The full class name of the '
+                                          'volume API class to use'),
 ]
 
-oslo_config.cfg.CONF.register_opts(_volume_opts)
+oslo.config.cfg.CONF.register_opts(_volume_opts)
 
 
 def API():
-    volume_api_class = oslo_config.cfg.CONF.volume_api_class
+    importutils = nova.openstack.common.importutils
+    volume_api_class = oslo.config.cfg.CONF.volume_api_class
     cls = importutils.import_class(volume_api_class)
     return cls()
